﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Konverter.NodeGenerators;
using Konverter.Statements;

namespace Konverter.CodeTree
{
    /// <summary>
    /// Datenstruktur für einen Baum, der die aus Xaml extrahierten und konvertierten Elemente und Daten enthält.
    /// Es gibt 3 konkrete Arten von Baumknoten:
    /// 1.) ChildElementNode: stellt einen Baumknoten dar, der im C#-Code durch ein eigenes Objekt wider gespiegelt wird
    ///     und beliebig viele Kindknoten sowie eine Liste von Attributknoten haben kann.
    /// 2.) PropertyValueNode: Knoten, der ein Xaml-Attribut mit Name und Wert speichert und im C#-Code eine an
    ///     eine Variable zugewiesene Eigenschaft darstellt.
    /// 3.) PropertyElementNode: Mischling eines ElementNode und eines PropertyNode. Es wird zunächst ein Property
    ///     gespeichert, welches so auch im C#-Code gesetz wird, allerdings ist der gespeicherte Inhalt nicht nur ein
    ///     Wert, sondern ein weiterer ChildElementNode, der wiederum einen beliebigen Unterbaum enthalten kann.
    /// </summary>
    class CSTree
    {
        /// <summary>
        /// Wurzel des Baumes
        /// </summary>
        ChildElementNode root;

        /// <summary>
        /// Konstruktor. Erstellt einen neuen Baum für die im übergebenen XmlDocument enthaltenen Xaml-Daten.
        /// </summary>
        /// <param name="xdoc">XmlDocument, aus dem der Baum erstellt werden soll.</param>
        public CSTree(XmlDocument xdoc)
        {
            // schnell die ungewollten Namespace-Attribute entfernen
            xdoc.FirstChild.Attributes.Remove(xdoc.FirstChild.Attributes["xmlns"]);
            xdoc.FirstChild.Attributes.Remove(xdoc.FirstChild.Attributes["xmlns:x"]);

            // Rücksetzen aller Variablenindizes
            NodeGeneratorManager.Instance.ResetIndexCounters();

            // wenn Kindknoten vorhanden sind: erstelle die Wurzel und daraus rekursiv den ganzen Baum
            if (xdoc.HasChildNodes)
            {
                try
                {
                    root = new ChildElementNode(xdoc.ChildNodes[0] as XmlElement, null);
                }
                catch (NotImplementedException exc)
                {
                    Logging.Instance.AddMessage(exc.Message);
                }
            }
        }

        /// <summary>
        /// Diese Methode liefert aus dem erstellten Baum eine komplette C#-Klasse "GUI" mit dem
        /// aus Xaml übersetzten Code für die Gui-Erstellung zurück. Darin enthalten sind auch alle
        /// Usings, die vom Code benötigt werden.
        /// </summary>
        /// <returns></returns>
        public string GetCSCode()
        {
            string code = "";

            // Code aus dem Baum erzeugen
            CSCodeStorage cscode = root.CreateCode();
            
            // ein paar statische usings, die anderen werden dynamisch erzeugt
            string usings =
                  "using System;\n"
                + "using System.Text;\n";

            // Rahmen-Code
            code += "class GUI : Window {\n\n";

            // Deklarationen für Member-Variablen
            if (cscode.GetMemberDecls().Count > 0)
            {
                int pad = 4;
                code += cscode.GenerateMemberDeclsCode(pad);
                code += "\n";
            }

            // weiterer Rahmencode
            code +=
                  "    [STAThread]\n"
                + "    public static void Main() {\n"
                + "             Application app = new Application();\n"
                + "             app.Run(new GUI());\n"
                + "    }\n\n"
                + "    public GUI(){\n";
            // Padding einer Codezeile (Anzahl Leerzeichen links)
            int padding = 8;
            
            // Usings erzeugen
            usings = usings + cscode.GenerateUsingsCode() + "\n";
            // Elementcode erzeugen
            code = usings + code + cscode.GenerateElemStmtCode(padding);
            // Add-Code erzeugen (Code zum Hinzufügen von Elementen zu anderen Elementen)
            code = code + "\n" + cscode.GenerateAddStmtCode(padding);
            
            // Rahmen schließen
            code +=
                  "\n"
                + "        Content =" + root.VarName + ";\n"
                + "    }\n\n"
                + "}";
            
            return code;
        }
    }
}
