﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.NodeGenerators;
using Konverter.Statements;
using System.Xml;
using Konverter.ConverterBase;
using Konverter.AttribValueConverters;

namespace Konverter.CodeTree
{
    class ChildElementNode : ElementNode
    {
        ElementNode parent;                      // Vaterknoten
        List<ElementNode> children;              // Kind-Elementknoten
        List<PropertyValueNode> attribs;         // Gespeicherte Werte-Attributknoten
        string varName;                          // C#-Variablenname dieses Knotens
        NodeGenerator nodeGenerator;             // NodeGenerator, der für diesen Knoten steht
        bool isMember;                           // true, wenn die Variable Klassenmember ist
        
        /// <summary>
        /// Variablenname dieses Knotens, womit im C#-Code ein eigenständiges Objekt erzeugt wird.
        /// </summary>
        public string VarName
        {
            get { return varName; }
        }

        /// <summary>
        /// Kind-Elementknoten, für die eine rekursive Code-Erzeugung stattfindet und die
        /// dem aus diesem Knoten erzeugten Objekt im C#-Code als eigenständige Kind-Objekte
        /// hinzugefügt werden.
        /// </summary>
        public List<ElementNode> Children
        {
            get { return children; }
        }

        /// <summary>
        ///  Vaterknoten, == null für die Baumwurzel.
        /// </summary>
        public ElementNode Parent
        {
            get { return parent; }
        }

        /// <summary>
        /// NodeGenerator, der für diesen Knoten zuständig ist.
        /// </summary>
        public NodeGenerator NodeGenerator
        {
            get { return nodeGenerator; }
        }

        /// <summary>
        /// Konstruktor.
        /// </summary>
        /// <param name="node">Xaml-Knoten, aus dem der Baumknoten erstellt werden soll</param>
        /// <param name="parent">Vaterknoten im CSTree</param>
        public ChildElementNode(XmlElement node, ElementNode parent)
        {
            this.parent = parent;
            this.children = new List<ElementNode>();

            // Variablenname speichern, falls das Attribut "Name" in der Attributliste auftauchen sollte
            ElemAttribute nameAttrib = null;

            // 1.) NodeGenerator erhalten, der für den Xaml-Knoten zuständig ist
            this.nodeGenerator = NodeGeneratorManager.Instance.GetGeneratorForXmlNode(node);
            if (this.nodeGenerator == null)
                throw new NotImplementedException("Noch keine Elementbehandlung für: " + node.Name);

            // 2.) XmlAttribute in Attributliste umwandeln
            List<ElemAttribute> valueAttribs = new List<ElemAttribute>();
            foreach (XmlAttribute attrib in node.Attributes)
            {
                // wenn Punkt im Attributnamen -> suche nach Tag in NodeGenerator-Hierarchie
                string attribName = attrib.Name;
                int iPoint = attribName.IndexOf('.');
                if (iPoint > 0)
                {
                    // wenn Tag in NodeGenerator-Hierarchie vorhanden -> setze Attributnamen um
                    if (nodeGenerator.HasTagInHierarchy(attribName.Substring(0, iPoint)))
                        attribName = attribName.Substring(iPoint + 1);
                    else
                    {
                        Logging.Instance.AddMessage("Attribut " + attrib.Name + ": " + attribName.Substring(0, iPoint) + " ist nicht in der Objekt-Hierarchie von " + node.Name + " enthalten");
                        attribName = default(string);
                    }
                }

                // setze Attribut
                if (attribName != default(string))
                {
                    // erzeuge Attribut
                    ElemAttribute newAtt = new ElemAttribute(attribName, attrib.Value);
                    valueAttribs.Add(newAtt);
                    if (newAtt.Name == "Name")
                        nameAttrib = newAtt;
                }
            }

            // 3.) Xml-Kindliste vollständig zerlegen in Elemente und Attribute
            List<XmlElement> elemAttribs = new List<XmlElement>();
            List<XmlElement> elemChildren = new List<XmlElement>();
            foreach (XmlNode childNode in node.ChildNodes)
            {
                // wenn der Xml-Knoten als XmlElement gesehen werden kann
                if (childNode is XmlElement)
                {
                    XmlElement child = (XmlElement)childNode;
                    // wenn das Kind einen Punkt enthält und vor dem Punkt ein Tag der aktuellen
                    // NodeGenerator-Hierarchie steht -> Attributknoten
                    int iPoint = child.Name.IndexOf('.');
                    if (iPoint>0)
                    {
                        if (nodeGenerator.HasTagInHierarchy(child.Name.Substring(0, iPoint)))
                        {
                            // wenn das Kind nur Text als Inhalt hat -> neues Werte-Attribut
                            if (child.ChildNodes.Count == 1 && child.ChildNodes[0] is XmlText)
                            {
                                ElemAttribute newAtt = new ElemAttribute(child.Name.Substring(iPoint + 1), child.ChildNodes[0].Value);
                                valueAttribs.Add(newAtt);
                                if (newAtt.Name == "Name")
                                    nameAttrib = newAtt;
                            }
                            // ansonsten steht das Attribut als eigenes Element
                            else
                            {
                                elemAttribs.Add(child);
                            }
                        }
                        else
                            Logging.Instance.AddMessage("Attributknoten " + child.Name + ": " + child.Name.Substring(0, iPoint) + " ist nicht in der Objekt-Hierarchie von " + node.Name + " enthalten");
                    }
                    // sonst: neuer Elementknoten
                    else
                    {
                        elemChildren.Add(child);
                    }
                }
                // wenn der Xml-Knoten als Text definiert ist
                else if (childNode is XmlText)
                {
                    // füge das Kind als Attribut hinzu, falls es wie ein Attribut behandelt werden soll
                    if (nodeGenerator is TextAsAttribHandler)
                        valueAttribs.Add(new ElemAttribute((nodeGenerator as TextAsAttribHandler).GetTextAttribName(), childNode.Value));
                    // füge das Kind als eigenen Knoten hinzu, falls es wie ein Kindknoten behandelt werden soll
                    else if (nodeGenerator is TextAsChildItemHandler)
                    {
                        try
                        {
                            TextAsChildItemHandler handler = nodeGenerator as TextAsChildItemHandler;
                            ChildElementNode newNode = new ChildElementNode(handler.GetItemName(), handler.GetAttribName(), childNode.Value, this);
                            this.children.Add(newNode);
                        }
                        catch (NotImplementedException exc)
                        {
                            Logging.Instance.AddMessage(exc.Message);
                        }
                    }
                    // ansonsten: wird nicht behandelt, daher Meldung zum Log hinzufügen
                    else
                        Logging.Instance.AddMessage("Noch keine Textbehandlung für das Element " + node.Name);
                }
            }

            // 4.) Attribut-Kindknoten des Baums berechnen (über die Hierarchie des NodeGenerators)
            this.attribs = nodeGenerator.GenerateAttribNodes(valueAttribs);

            // 5.) Variablenname erzeugen
            this.varName = nodeGenerator.GenCurrentVarName(nameAttrib);
            this.isMember = (nameAttrib != null);

            // 6.) Kindknoten für jeden Elementknoten erzeugen und an Kindliste anhängen
            // 6.1) Elementattribute
            foreach (XmlElement xmlChild in elemAttribs)
            {
                try
                {
                    PropertyElementNode child = new PropertyElementNode(xmlChild.Name.Substring(node.Name.Length + 1), xmlChild, this);
                    this.children.Add(child);
                }
                catch (NotImplementedException exc)
                {
                    Logging.Instance.AddMessage(exc.Message);
                }
            }
            // 6.2) Kind-Elementknoten
            foreach (XmlElement xmlChild in elemChildren)
            {
                // versuche den Kindknoten zu erzeugen; eine Exception wird geworfen, wenn der Name
                // des Kindknoten nicht behandelt werden kann, z.B. weil die Implementierung dafür fehlt
                try
                {
                    ChildElementNode child = new ChildElementNode(xmlChild, this);
                    this.children.Add(child);
                }
                catch (NotImplementedException exc)
                {
                    Logging.Instance.AddMessage(exc.Message);
                }
            }
        }

        /// <summary>
        /// Konstruktor für direkte Erzeugung eines ChildElementNode mit einem Attribut.
        /// </summary>
        /// <param name="itemName">Name der Klasse/des Xml-Tags für diesen Knoten</param>
        /// <param name="attribName">Name des zu setzenden Attributs</param>
        /// <param name="attribValue">Wert des zu setzenden Attributs</param>
        /// <param name="parent">Elternknoten</param>
        public ChildElementNode(string itemName, string attribName, string attribValue, ChildElementNode parent)
        {
            this.parent = parent;
            this.children = new List<ElementNode>();
            // Attribut erzeugen
            this.attribs = new List<PropertyValueNode>() { new PropertyValueNode(attribName, AttStringConverter.Convert(attribValue)) };
            // NodeGenerator holen
            this.nodeGenerator = NodeGeneratorManager.Instance.GetGeneratorForTag(itemName);
            if (this.nodeGenerator == null)
                throw new NotImplementedException("Noch keine Elementbehandlung für: " + itemName);
            // Variablenname erzeugen
            this.varName = nodeGenerator.GenCurrentVarName();
        }

        /// <summary>
        /// Erstellt den C#-Code für dieses Element und rekursiv für den ganzen untergeordneten Baum.
        /// </summary>
        /// <returns>C#-Code des Knotens sowie des Unterbaumes</returns>
        public CSCodeStorage CreateCode()
        {
            // C#-Code für ein neues Objekt mit dem gespeicherten Variablennamen
            List<Statement> elemCode = new List<Statement>();
            List<Assignment> memberDecls = new List<Assignment>();
            Assignment newObjDecl = nodeGenerator.GenNewObjectStmt(this.varName);
            // wenn Member, dann in Member-Deklarationen speichern, ansonsten im Elementcode
            if (isMember)
                memberDecls.Add(newObjDecl);
            else
                elemCode.Add(newObjDecl);

            // Code für diesen Knoten erhalten
            elemCode.AddRange(getNodeElemCode());          // ElementCode generieren
            List<Statement> addCode = getNodeAddCode();    // AddCode generieren
            ListSet<UsingStmt> usings = getNodeUsings();   // Usings dieses Knotens erhalten
            
            // Code von Kindern erhalten
            CSCodeStorage childCSCode = new CSCodeStorage();
            List<ChildElementNode> elemChildren = new List<ChildElementNode>();
            foreach (ElementNode child in children)
            {
                // Behandlung der PropertyElementNode's
                if (child is PropertyElementNode)
                    childCSCode.AppendCodeStorage(child.CreateCode());
                else  // ChildElementNode's an Extra-Liste anhängen -> werden gesondert betrachtet
                    elemChildren.Add(child as ChildElementNode);
            }
            // Behandlung er ChildElementNode's
            foreach (ChildElementNode child in elemChildren)
                childCSCode.AppendCodeStorage(child.CreateCode());

            // Codes in richtiger Reihenfolge vereinigen
            CSCodeStorage cscode = new CSCodeStorage();
            cscode.AppendElemStatements(elemCode);
            cscode.AppendElemStatements(childCSCode.GetElemStatements());
            cscode.AppendAddStatements(childCSCode.GetAddStatements());
            cscode.AppendAddStatements(addCode);
            cscode.AddUsings(usings);
            cscode.AddUsings(childCSCode.GetUsings());
            cscode.AppendMemberDecls(memberDecls);
            cscode.AppendMemberDecls(childCSCode.GetMemberDecls());

            return cscode;
        }

        /// <summary>
        /// Erzeugt den ElementCode für diesen Knoten.
        /// </summary>
        /// <returns>ElementCode des Knotens</returns>
        private List<Statement> getNodeElemCode()
        {
            List<Statement> elemCode = new List<Statement>();

            // Code für die Wertattribute erzeugen
            foreach (PropertyValueNode attrib in attribs)
            {
                if (attrib.RValue != null)       // null-Werte ignorieren -> Konvertierung schlug fehl
                    elemCode.Add(attrib.GetAttribStmt(this));
            }

            return elemCode;
        }

        /// <summary>
        /// Erzeugt den AddCode für diesen Knoten.
        /// </summary>
        /// <returns>AddCode des Knotens</returns>
        private List<Statement> getNodeAddCode()
        {
            List<ChildElementNode> elemChildren = new List<ChildElementNode>();
            // berechne zunächst die Liste der ChildElementNode's in children
            foreach (ElementNode child in children)
            {
                if (child is ChildElementNode)
                    elemChildren.Add(child as ChildElementNode);
            }

            // wenn es Kindelemente gibt -> füge diese im C#-Code zum Objekt dieses Knotens hinzu
            if (elemChildren.Count > 0)
            {
                // Söhne müssen zu einem Objekt dieses Knotens hinzugefügt werden dürfen, ansonsten Fehler ins Log
                if (nodeGenerator is ChildElementsAddHandler)
                    return (nodeGenerator as ChildElementsAddHandler).GenerateAddCode(this, elemChildren);
                else
                    Logging.Instance.AddMessage("Keine Behandlung für Kindknoten des Elements " + nodeGenerator.TagName);
            }
                
            return null;
        }

        /// <summary>
        /// Gibt die Menge der von diesem Knoten und seinen Attributen benötigten Usings zurück.
        /// </summary>
        /// <returns>Benötigte Usings.</returns>
        private ListSet<UsingStmt> getNodeUsings()
        {
            // Usings von diesem Knoten aus dem NodeGenerator erhalten
            ListSet<UsingStmt> usings = new ListSet<UsingStmt>(nodeGenerator.ElementUsings);
            // Benötigte Usings der Wertattribute erhalten
            foreach (PropertyValueNode attrib in attribs)
            {
                if (attrib.Usings != null)
                    usings.AddRange(attrib.Usings);
            }
            return usings;
        }
    }
}
