﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.NodeGenerators;
using Konverter.Statements;
using System.Xml;

namespace Konverter.CodeTree
{
    /// <summary>
    /// Baumknoten des Typs PropertyElementNode sind "Zwitter" zwischen ElementNode und PropertyNode.
    /// Sie sind PropertyNode's in dem Sinne, dass in ihnen der Name einer Property gespeichert wird 
    /// und bei der Umwandlung in C#-Code diese Property in der Variable des Elternknotens gesetzt wird.
    /// Sie sind aber auch ElementNode's in dem Sinn, dass sie keine Blätter des Baumes darstellen, sondern
    /// als Unterelemente wiederum einen beliebig großen Baum haben können. Sie besitzen also 
    /// entgegen den PropertyValueNode's nicht nur einen einzelnen Wert, sondern eine Hierarchie von
    /// Objekten, wobei bei der Code-Erzeugung das oberste Objekt dieser Hierarchie in der Property
    /// gesetzt wird.
    /// </summary>
    class PropertyElementNode : ElementNode, PropertyNode
    {
        ElementNode parent;                 // Vaterknoten
        List<ChildElementNode> children;    // Kindknoten (wenn Collection, dann mehrere)
        string attribName;                  // Name des Attributs
        bool isCollection;                  // true, wenn das Attribut eine Collection darstellt

        /// <summary>
        /// Gibt den Namen der der ersten Kind-Variablen zurück.
        /// </summary>
        public string VarName
        {
            get { return children[0].VarName; }
        }

        /// <summary>
        /// Gibt den Namen des Xaml-Attributs zurück, welches in diesem Knoten gespeichert wird.
        /// </summary>
        public string AttribName
        {
            get { return attribName; }
        }

        /// <summary>
        /// Gibt die Kindknoten zurück.
        /// </summary>
        public List<ChildElementNode> Children
        {
            get { return children; }
        }

        /// <summary>
        /// Vaterknoten im CSTree.
        /// </summary>
        public ElementNode Parent
        {
            get { return parent; }
        }

        /// <summary>
        /// NodeGenerator, der für den Knoten steht. Gibt hier den für die Kindknoten interessanten
        /// NodeGenerator des Vaterknotens zurück.
        /// </summary>
        public NodeGenerator NodeGenerator
        {
            get { return parent.NodeGenerator; }
        }

        /// <summary>
        /// Konstruktor.
        /// </summary>
        /// <param name="attribName">Attributname</param>
        /// <param name="node">Xaml-Knoten, aus dessen Kindern dieser Baumknoten erstellt werden soll</param>
        /// <param name="parent">Vaterknoten im CSTree</param>
        public PropertyElementNode(string attribName, XmlElement node, ElementNode parent)
        {
            this.attribName = attribName;
            this.parent = parent;
            
            // mind. 1 Kindknoten muss vorhanden sein
            if (node.ChildNodes.Count == 0)
                throw new NotImplementedException("Der Attributknoten " + node.Name + " hat keine Kindknoten");

            // 1.) ermitteln, ob das Attribut eine Collection darstellt
            this.isCollection = this.parent.NodeGenerator.CollectionAttribs.Contains(this.attribName);

            // 2.) wenn keine Collection, dann darf es nur ein Kind geben
            if (!isCollection && node.ChildNodes.Count > 1)
                Logging.Instance.AddMessage("Der Attributknoten " + node.Name + " besitzt mehr als 1 Kindknoten");

            // 3.) Kinder hinzufügen
            this.children = new List<ChildElementNode>();
            foreach (XmlElement child in node.ChildNodes)
                this.children.Add(new ChildElementNode(child, this));
        }

        /// <summary>
        /// Erstellt den C#-Code aus sich und rekursiv aus dem gespeicherten Unterbaum.
        /// </summary>
        /// <returns></returns>
        public CSCodeStorage CreateCode()
        {
            // rekursiver Aufruf für den Unterbaum
            CSCodeStorage cscode = new CSCodeStorage();
            foreach(ChildElementNode child in this.children)
                cscode.AppendCodeStorage(child.CreateCode());
            // und hier eine Zuweisung zum Hinzufügen der Kinder als Property im C#-Code erzeugen
            // wenn Collection, dann zum AddCode, ansonsten zum ElementCode
            if(isCollection)
                cscode.AppendAddStatements(getAddStatements());
            else
                cscode.AppendElemStatements(getAddStatements());

            return cscode;
        }

        /// <summary>
        /// Gibt eine Liste von Anweisungen zurück, welche das Hinzufügen der Variablen der Kindknotens
        /// als Property der Variable des Elternknotens beinhaltet.
        /// </summary>
        /// <returns>Anweisungsliste</returns>
        private List<Statement> getAddStatements()
        {
            List<Statement> stmts = new List<Statement>();
            
            // wenn keine Collection, dann erstes Kind direkt zuweisen
            if (!isCollection)
            {
                stmts.Add(new Assignment(parent.VarName + "." + attribName, children[0].VarName));
            }
            // ansonsten: alle Kinder als Collection per ".Add()" zuweisen
            else
            {
                foreach (ChildElementNode child in this.children)
                    stmts.Add(new MethodCall(parent.VarName + "." + attribName + ".Add", child.VarName));
            }

            return stmts;
        }
    }
}
