﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.Statements;

namespace Konverter.CodeTree
{
    /// <summary>
    /// Ein PropertyValueNode entsteht aus einem Attribut eines Xaml-Knotens, indem der Name des
    /// Attributs gespeichert wird und zusätzlich noch der Wert des Attributs in einen C#-konformen
    /// Wert überführt und gespeichert wird.
    /// Im erstellten C#-Code wird ein PropertyValueNode durch eine Eigenschaft (Property) eines
    /// Objekts dargestellt, an welches diese zugewiesen wird.
    /// </summary>
    class PropertyValueNode : PropertyNode
    {
        string attribName = "";                  // Attributname
        string rvalue = "";                      // Konvertierter Attributwert
        ListSet<UsingStmt> usings = null;        // benötigte Usings

        /// <summary>
        /// Gibt den Namen des gespeicherten Attributs zurück.
        /// </summary>
        public string AttribName
        {
            get { return attribName; }
        }

        // Gibt den C#-konformen Wert des gespeicherten Attributs zurück.
        public string RValue
        {
            get { return rvalue; }
        }

        /// <summary>
        /// Gibt die Liste der benötigten Usings zurück.
        /// </summary>
        public ListSet<UsingStmt> Usings
        {
            get { return usings; }
        }


        /// <summary>
        /// Konstruktor.
        /// </summary>
        /// <param name="name">Attributname</param>
        /// <param name="rvalue">C#-konformer Attributwert als string</param>
        public PropertyValueNode(string name, string rvalue)
        {
            this.attribName = name.Trim();
            this.rvalue = (rvalue == null) ? null : rvalue.Trim();
        }


        /// <summary>
        /// Konstruktor.
        /// </summary>
        /// <param name="name">Attributname</param>
        /// <param name="rvalue">C#-konformer Attributwert als string</param>
        /// <param name="usings">Menge benötigter Usings für die Eigenschaft</param>
        public PropertyValueNode(string name, string rvalue, ListSet<UsingStmt> usings) 
            : this(name, rvalue)
        {
            this.usings = usings;
        }

        /// <summary>
        /// Konstruktor.
        /// </summary>
        /// <param name="name">Attributname</param>
        /// <param name="rvalue">C#-konformer Attributwert als string</param>
        /// <param name="usings">Benötigtes Using für die Eigenschaft</param>
        public PropertyValueNode(string name, string rvalue, UsingStmt usingStmt)
            : this(name, rvalue, new ListSet<UsingStmt>() { usingStmt }) { }

        /// <summary>
        /// Konstruktor.
        /// </summary>
        /// <param name="name">Attributname</param>
        /// <param name="rvalue">C#-konformer Attributwert als string</param>
        /// <param name="usings">Liste benötigter Usings für die Eigenschaft</param>
        public PropertyValueNode(string name, string rvalue, List<UsingStmt> usings)
            : this(name, rvalue)
        {
            this.usings = new ListSet<UsingStmt>(usings);
        }

        /// <summary>
        /// Gibt den C#-Code des Attributs zurück. Dieser enthält die benötigten Usings
        /// sowie den Code zum Setzen der Eigenschaft in ihrem Element.
        /// </summary>
        /// <param name="parent">Vaterknoten, an dessen Variable die Eigenschaft gesetzt werden soll.</param>
        /// <returns>C#-Code des Attributs</returns>
        public CSCodeStorage GetAttribCode(ElementNode parent)
        {
            CSCodeStorage cscode = new CSCodeStorage();
            if(rvalue != null)
                cscode.AppendElemStatement(GetAttribStmt(parent));
            cscode.AddUsings(usings);
            return cscode;
        }

        /// <summary>
        /// Gibt die Anweisung zurück, welche die Eigenschaft bei der Variablen des Vaterknotens setzt.
        /// </summary>
        /// <param name="parent">Vaterknoten</param>
        /// <returns>Anweisung des Hinzufügens der Eigenschaft oder null, wenn der Attributwert null ist</returns>
        public Statement GetAttribStmt(ElementNode parent)
        {
            if (rvalue == null)
                return null;
            else
                return new Assignment(parent.VarName + "." + attribName, rvalue);
        }
    }
}
