﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.Statements;
using System.Collections;

namespace Konverter
{
    /// <summary>
    /// Speicherstruktur für gekapselte Daten von erzeugtem Code.
    /// Es werden gespeichert:
    /// 1.) ElemStatements: Eine Liste von Anweisungen (Statements), die den "Elementcode" (Erzeugen von Variablen
    ///     und setzen von Eigenschaften) in linearer Form enthält.
    /// 2.) AddCode: Liste von Statements, die den "Hinzufüge-Code" (Hinzufügen von Kindelementen zu einer Variable
    ///     z.B. durch "Items.Add()" gesondert speichert. Dadurch erhält man eine bessere Strukturierung
    ///     des erzeugten Codes.
    /// 3.) Usings: Eine Menge von using's, die vom erzeugten C#-Code benötigt werden.
    /// 4.) MemberDecls: Deklarationen von Klassen-Member-Variablen.
    /// </summary>
    class CSCodeStorage
    {
        /// <summary>
        /// Liste von Anweisungen (= Code in gekapselter Form) --> Element-Code
        /// </summary>
        List<Statement> elemStmts = new List<Statement>();
        /// <summary>
        /// Liste von Anweisungen (= Code in gekapselter Form) --> Hinzufüge-Code
        /// </summary>
        List<Statement> addStmts = new List<Statement>();
        /// <summary>
        /// Menge von using-Direktiven, die vom erzeugten Code benötigt werden.
        /// </summary>
        Set<UsingStmt> usings = new ListSet<UsingStmt>();
        /// <summary>
        /// Liste von Zuweisungen (Erzeugung von Member-Variablen).
        /// </summary>
        List<Assignment> memberDecls = new List<Assignment>();

        /// <summary>
        /// Default-Konstruktor.
        /// </summary>
        public CSCodeStorage() { }

        /// <summary>
        /// Gibt die Liste der ElementCode-Anweisungen zurück.
        /// </summary>
        /// <returns>Anweisungs-Liste</returns>
        public List<Statement> GetElemStatements()
        {
            return elemStmts;
        }

        /// <summary>
        /// Gibt die Liste der AddCode-Anweisungen zurück.
        /// </summary>
        /// <returns>Anweisungs-Liste</returns>
        public List<Statement> GetAddStatements()
        {
            return addStmts;
        }

        /// <summary>
        /// Gibt die gespeicherte Menge von Usings zurück.
        /// </summary>
        /// <returns>Anweisungs-Liste</returns>
        public Set<UsingStmt> GetUsings()
        {
            return usings;
        }

        /// <summary>
        /// Gibt die Liste von Zuweisungen zurück, welche dei Deklarationen
        /// von Member-Variablen enthält.
        /// </summary>
        /// <returns>Zuweisungen</returns>
        public List<Assignment> GetMemberDecls()
        {
            return memberDecls;
        }

        /// <summary>
        /// Fügt eine ElementCode-Anweisung der Speicherstruktur hinzu.
        /// </summary>
        /// <param name="stmt">Anweisung</param>
        public void AppendElemStatement(Statement stmt)
        {
            if (stmt != null)
                elemStmts.Add(stmt);
        }

        /// <summary>
        /// Fügt mehrere ElementCode-Anweisungen der Speicherstruktur hinzu.
        /// </summary>
        /// <param name="stmts">Anweisungen</param>
        public void AppendElemStatements(IEnumerable<Statement> stmts)
        {
            if (stmts != null)
                elemStmts.AddRange(stmts);
        }

        /// <summary>
        /// Fügt eine AddCode-Anweisung der Speicherstruktur hinzu.
        /// </summary>
        /// <param name="stmt">Anweisung</param>
        public void AppendAddStatement(Statement stmt)
        {
            if (stmt != null)
                addStmts.Add(stmt);
        }

        /// <summary>
        /// Fügt mehrere AddCode-Anweisung der Speicherstruktur hinzu.
        /// </summary>
        /// <param name="stmts">Anweisung</param>
        public void AppendAddStatements(IEnumerable<Statement> stmts)
        {
            if (stmts != null)
                addStmts.AddRange(stmts);
        }

        /// <summary>
        /// Fügt ein Using der Speicherstruktur hinzu.
        /// </summary>
        /// <param name="usg">using</param>
        public void AddUsing(UsingStmt usg)
        {
            if(usg != null)
                usings.Add(usg);
        }

        /// <summary>
        /// Fügt mehrere Usings der Speicherstruktur hinzu.
        /// </summary>
        /// <param name="usgs">usings</param>
        public void AddUsings(IEnumerable<UsingStmt> usgs)
        {
            if (usgs != null)
                usings.AddRange(usgs);
        }

        /// <summary>
        /// Fügt der Liste der Member-Variablen-Deklarationen eine neue Deklaration hinzu.
        /// </summary>
        /// <param name="decl">Hinzuzufügende Deklaration</param>
        public void AppendMemberDecl(Assignment decl)
        {
            if(decl != null)
                this.memberDecls.Add(decl);
        }

        /// <summary>
        /// Fügt der Liste der Member-Variablen-Deklarationen neue Deklarationen hinzu.
        /// </summary>
        /// <param name="decls">Hinzuzufügende Deklarationen</param>
        public void AppendMemberDecls(IEnumerable<Assignment> decls)
        {
            if(decls != null)
                this.memberDecls.AddRange(decls);
        }

        /// <summary>
        /// Fügt an die Speicherstruktur einen Speicher gleichen Typs an.
        /// </summary>
        /// <param name="code">Anzufügende Speicherstruktur</param>
        public void AppendCodeStorage(CSCodeStorage code)
        {
            AppendElemStatements(code.elemStmts);
            AppendAddStatements(code.addStmts);
            AddUsings(code.usings);
            AppendMemberDecls(code.memberDecls);
        }

        /// <summary>
        /// Erzeugt den ElementCode der gespeicherten Anweisungen mit 0 Leerzeichen links und gibt ihn zurück.
        /// </summary>
        /// <returns>Erzeugter Code</returns>
        public string GenerateElemStmtCode()
        {
            return GenerateElemStmtCode(0);
        }

        /// <summary>
        /// Erzeugt den ElementCode der gespeicherten Anweisungen mit "padding" Leerzeichen links.
        /// </summary>
        /// <param name="padding">Anzahl der Leerzeichen links</param>
        /// <returns>Erzeugter Code</returns>
        public string GenerateElemStmtCode(int padding)
        {
            return generateStmtCode(elemStmts, padding);
        }

        /// <summary>
        /// Erzeugt den AddCode der gespeicherten Anweisungen mit 0 Leerzeichen links und gibt ihn zurück.
        /// </summary>
        /// <returns>Erzeugter Code</returns>
        public string GenerateAddStmtCode()
        {
            return GenerateAddStmtCode(0);
        }

        /// <summary>
        /// Erzeugt den AddCode der gespeicherten Anweisungen mit "padding" Leerzeichen links.
        /// </summary>
        /// <param name="padding">Anzahl der Leerzeichen links</param>
        /// <returns>Erzeugter Code</returns>
        public string GenerateAddStmtCode(int padding)
        {
            return generateStmtCode(addStmts, padding);
        }

        /// <summary>
        /// Erzeugt den Code der Member-Variablen-Deklarationen mit 0 Leerzeichen links und gibt ihn zurück.
        /// </summary>
        /// <returns>Erzeugter Code</returns>
        public string GenerateMemberDeclsCode()
        {
            return GenerateMemberDeclsCode(0);
        }

        /// <summary>
        /// Erzeugt den Code der Member-Variablen-Deklarationen mit "padding" Leerzeichen links.
        /// </summary>
        /// <param name="padding">Anzahl der Leerzeichen links</param>
        /// <returns>Erzeugter Code</returns>
        public string GenerateMemberDeclsCode(int padding)
        {
            List<Statement> list = new List<Statement>();
            foreach (Assignment assign in memberDecls)
                list.Add(assign);
            return generateStmtCode(list, padding);
        }

        /// <summary>
        /// Fasst die Erzeugung von Statement-Code in einer privaten Methode zusammen.
        /// </summary>
        /// <param name="stmts">Anweisungen, aus denen Code erzeugt werden soll</param>
        /// <param name="padding">Anzahl an links einzufügenden Leerzeichen</param>
        /// <returns></returns>
        private string generateStmtCode(List<Statement> stmts, int padding)
        {
            string code = "";

            // padding-String erzeugen
            string pad = "";
            for (int i = 0; i < padding; i++)
                pad += " ";

            // Code für jede Anweisung erzeugen und hinzufügen
            foreach (Statement stmt in stmts)
            {
                code += pad + stmt.ToCode() + "\n";
            }

            return code;
        }

        /// <summary>
        /// Erzeugt den Code der gespeicherten Usings mit 0 Leerzeichen links und gibt diesen zurück.
        /// </summary>
        /// <returns>Erzeugter Code</returns>
        public string GenerateUsingsCode()
        {
            return GenerateUsingsCode(0);
        }

        /// <summary>
        /// Erzeugt den Code der gespeicherten Usings mit "padding" Leerzeichen links.
        /// </summary>
        /// <param name="padding">Anzahl der Leerzeichen links</param>
        /// <returns>Erzeugter Code</returns>
        public string GenerateUsingsCode(int padding)
        {
            string code = "";

            // padding-String erzeugen
            string pad = "";
            for (int i = 0; i < padding; i++)
                pad += " ";

            if (this.usings.Count > 0)
            {
                // wandle die Usings in ein Array um und sortiere dem Namen nach
                UsingStmt[] usgs = new UsingStmt[this.usings.Count];
                this.usings.CopyTo(usgs, 0);
                Array.Sort<UsingStmt>(usgs);

                // erzeuge Code für jedes Using
                for(int i = 0; i < usgs.Length; i++)
                {
                    code += pad + usgs[i].ToCode() + "\n";
                }
            }
            
            return code;
        }
    }
}
