﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Konverter.Statements;
using Konverter.CodeTree;
using Konverter.ConverterBase;
using Konverter.AttribValueConverters;

namespace Konverter.NodeGenerators.BaseGenerators
{
    /// <summary>
    /// Abstrakte Klasse für einen NodeGenerator, der Baumknoten für Elemente erzeugt,
    /// die vom Typ "ContentControl" abstammen.
    /// Dabei wird gleich die Schnittstelle "TextAsAttribHandler" mit implementiert,
    /// da bei allen ContentControl's XmlText über das Attribut "Content" behandelt
    /// werden soll.
    /// Außerdem wird die Schnittstelle "ChildElementsAddHandler" implementiert, um 
    /// Kindknoten einem ContentControl über die Methode "Content.Add()" hinzuügen zu können.
    /// </summary>
    abstract class BaseGenContentControl : BaseGenControl, TextAsAttribHandler, ChildElementsAddHandler
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public BaseGenContentControl() : base()
        {
            // füge Collection-Informationen hinzu
            collectionAttribs.Add("Content");
        }

        /// <summary>
        /// Erzeugt Baumknoten für die übergebenen Attribute, welche hier behandelt werden sollen.
        /// </summary>
        /// <returns>Erzeugte Baumknoten</returns>
        protected override List<PropertyValueNode> GenerateBaseNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();

            // alle Attribute durchgehen und die von ContentControl behandeln
            foreach (ElemAttribute attrib in attribs)
            {
                switch (attrib.Name)
                {
                    //object
                    case "Content":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttObjectConverter.Convert(attrib.Value)));
                        break;
                    
                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // rekursive Code-Erzeugung durch die Basisklasse (Control)
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if ("ContentControl" == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }

        public string GetTextAttribName()
        {
            return "Content";
        }

        public List<Konverter.Statements.Statement> GenerateAddCode(ChildElementNode parent, List<ChildElementNode> children)
        {
            List<Statement> addCode = new List<Statement>();
            foreach (ChildElementNode child in children)
                addCode.Add(new Assignment(parent.VarName + ".Content", child.VarName));
            return addCode;
        }

    }
}
