﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.Statements;
using System.Xml;
using Konverter.ConverterBase;
using Konverter.CodeTree;
using Konverter.AttribValueConverters;

namespace Konverter.NodeGenerators.BaseGenerators
{
    /// <summary>
    /// Abstrakte Klasse für einen NodeGenerator, der Baumknoten für Elemente erzeugt,
    /// die vom Typ "Control" abstammen.
    /// </summary>
    abstract class BaseGenControl : BaseGenFrameworkElement
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public BaseGenControl() : base()
        {
            
        }

        /// <summary>
        /// Erzeugt Baumknoten für die übergebenen Attribute, welche hier behandelt werden sollen.
        /// </summary>
        /// <returns>Erzeugte Baumknoten</returns>
        protected override List<PropertyValueNode> GenerateBaseNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();

            // Verarbeitung von Attributen, die direkt für jedes Control definiert sind
            foreach (ElemAttribute attrib in attribs)
            {
                switch (attrib.Name)
                {
                    //bool
                    case "IsTabStop":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttBoolConverter.Convert(attrib.Value)));
                        break;
                    
                    //int
                    case "TabIndex":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttIntConverter.Convert(attrib.Value)));
                        break;

                    //double
                    case "FontSize":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttDoubleConverter.Convert(attrib.Value)));
                        break;

                    //Alignment
                    case "HorizontalContentAlignment":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttHorizontalAlignmentConverter.Convert(attrib.Value), AttHorizontalAlignmentConverter.Usings));
                        break;
                    case "VerticalContentAlignment":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttVerticalAlignmentConverter.Convert(attrib.Value), AttVerticalAlignmentConverter.Usings));
                        break;
                    
                    //Brush
                    case "Background":
                    case "BorderBrush":
                    case "Foreground":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttBrushConverter.Convert(attrib.Value), AttBrushConverter.Usings));
                        break;

                    //FontFamily
                    case "FontFamily":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttFontFamilyConverter.Convert(attrib.Value), AttFontFamilyConverter.Usings));
                        break;

                    //FontStretch
                    case "FontStretch":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttFontStretchConverter.Convert(attrib.Value), AttFontStretchConverter.Usings));
                        break;
                    
                    //FontWeight
                    case "FontWeight":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttFontWeightConverter.Convert(attrib.Value), AttFontWeightConverter.Usings));
                        break;
                    
                    //Thickness
                    case "BorderThickness":
                    case "Padding":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttThicknessConverter.Convert(attrib.Value), AttThicknessConverter.Usings));
                        break;

                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // rekursive Code-Erzeugung durch die Basisklasse (FrameworkElement)
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if ("Control" == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }
    }
}
