﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Konverter.Statements;
using Konverter.ConverterBase;
using Konverter.CodeTree;
using Konverter.AttribValueConverters;

namespace Konverter.NodeGenerators.BaseGenerators
{
    /// <summary>
    /// Abstrakte Klasse für einen NodeGenerator, der Baumknoten für Elemente erzeugt,
    /// die vom Typ "FrameworkElement" abstammen.
    /// </summary>
    abstract class BaseGenFrameworkElement : BaseGenUIElement
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public BaseGenFrameworkElement() : base()
        {
            
        }

        /// <summary>
        /// Erzeugt Baumknoten für die übergebenen Attribute, welche hier behandelt werden sollen.
        /// </summary>
        /// <returns>Erzeugte Baumknoten</returns>
        protected override List<PropertyValueNode> GenerateBaseNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();

            // Behandlung der Attribute, die in FrameworkElement definiert sind
            foreach (ElemAttribute attrib in attribs)
            {
                switch (attrib.Name)
                {
                    //bool
                    case "ForceCursor":
                    case "OverridesDefaultStyle":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttBoolConverter.Convert(attrib.Value)));
                        break;

                    //double
                    case "Height":
                    case "MaxHeight":
                    case "MaxWidth":
                    case "MinHeight":
                    case "MinWidth":
                    case "Width":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttLengthConverter.Convert(attrib.Value)));
                        break;
                    
                    //object
                    case "DataContext":
                    case "Tag":
                    case "ToolTip":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttObjectConverter.Convert(attrib.Value)));
                        break;

                    //string
                    case "Name":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttStringConverter.Convert(attrib.Value)));
                        break;


                    //Alignment
                    case "HorizontalAlignment":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttHorizontalAlignmentConverter.Convert(attrib.Value), AttHorizontalAlignmentConverter.Usings));
                        break;
                    case "VerticalAlignment":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttVerticalAlignmentConverter.Convert(attrib.Value), AttVerticalAlignmentConverter.Usings));
                        break;

                    //Cursor
                    case "Cursor":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttCursorConverter.Convert(attrib.Value), AttCursorConverter.Usings));
                        break;

                    //FlowDirection
                    case "FlowDirection":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttFlowDirectionConverter.Convert(attrib.Value), AttFlowDirectionConverter.Usings));
                        break;
                    
                    //Thickness
                    case "Margin":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttThicknessConverter.Convert(attrib.Value), AttThicknessConverter.Usings));
                        break;

                    //XmlLanguage
                    case "Language":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttXmlLanguageConverter.Convert(attrib.Value), AttXmlLanguageConverter.Usings));
                        break;
                    
                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // rekursive Code-Erzeugung durch die Basisklasse (UIElement)
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if ("FrameworkElement" == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }
    }
}
