﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Konverter.Statements;
using Konverter.ConverterBase;
using Konverter.CodeTree;
using Konverter.AttribValueConverters;

namespace Konverter.NodeGenerators.BaseGenerators
{
    /// <summary>
    /// Abstrakte Klasse für einen NodeGenerator, der Baumknoten für Elemente erzeugt,
    /// die vom Typ "Panel" abstammen.
    /// Dabei wird gleich an dieser Stelle die Schnittstelle "ChildElementsAddHandler"
    /// implementiert, da zu jedem Panel Kindelemente per "Children.Add()" hinzugefügt werden.
    /// </summary>
    abstract class BaseGenPanel : BaseGenFrameworkElement, ChildElementsAddHandler
    {

        /// <summary>
        /// Konstruktor.
        /// </summary>
        public BaseGenPanel() : base()
        {
            // füge Collection-Informationen hinzu
            collectionAttribs.Add("Children");
        }

        /// <summary>
        /// Erzeugt Baumknoten für die übergebenen Attribute, welche hier behandelt werden sollen.
        /// </summary>
        /// <returns>Erzeugte Baumknoten</returns>
        protected override List<PropertyValueNode> GenerateBaseNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();

            // Behandlung der Attribute, die direkt in Panel definiert sind
            foreach (ElemAttribute attrib in attribs)
            {
                switch (attrib.Name)
                {
                    //bool
                    case "IsItemsHost":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttBoolConverter.Convert(attrib.Value)));
                        break;

                    //Brush
                    case "Background":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttBrushConverter.Convert(attrib.Value), AttBrushConverter.Usings));
                        break;
                    
                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // rekursive Code-Erzeugung durch die Basisklasse (FrameworkElement)
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Methoden-Implementierung der Schnittstelle ChildElementsAddHandler.
        /// Diese Methode erzeugt aus den übergebenen Kindknoten eine Liste von
        /// Anweisungen, welche die Variablen der Kindknoten über eine definierte
        /// Methode der Variablen des Elternknotens hinzufügen.
        /// </summary>
        /// <param name="parent">Elternknoten</param>
        /// <param name="children">Kind-Elementknoten von parent</param>
        /// <returns></returns>
        public List<Statement> GenerateAddCode(ChildElementNode parent, List<ChildElementNode> children)
        {
            List<Statement> addCode = new List<Statement>();
            // für alle Kinder einen neuen Methodenaufruf erstellen, der sie der Vatervariable hinzufügt
            foreach(ChildElementNode child in children)
                addCode.Add(new MethodCall(parent.VarName + ".Children.Add", child.VarName));
            return addCode;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if ("Panel" == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }
    }
}
