﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Konverter.Statements;
using Konverter.ConverterBase;
using Konverter.CodeTree;
using Konverter.AttribValueConverters;

namespace Konverter.NodeGenerators.BaseGenerators
{
    /// <summary>
    /// Abstrakte Klasse für einen NodeGenerator, der Baumknoten für Elemente erzeugt,
    /// die vom Typ "UIElement" abstammen.
    /// </summary>
    abstract class BaseGenUIElement : NodeGenerator
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public BaseGenUIElement() : base()
        {
            
        }

        /// <summary>
        /// Erzeugt Baumknoten für die übergebenen Attribute, welche hier behandelt werden sollen.
        /// </summary>
        /// <returns>Erzeugte Baumknoten</returns>
        protected override List<PropertyValueNode> GenerateBaseNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();

            // Verarbeitung der Attribute, die allen UIElementen gemein sind
            foreach (ElemAttribute attrib in attribs)
            {
                switch (attrib.Name)
                {
                    //bool
                    case "AllowDrop":
                    case "ClipToBounds":
                    case "Focusable":
                    case "IsEnabled":
                    case "IsHitTestVisible":
                    case "SnapsToDevicePixels":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttBoolConverter.Convert(attrib.Value)));
                        break;

                    //double
                    case "Opacity":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttDoubleConverter.Convert(attrib.Value)));
                        break;

                    //Brush
                    case "OpacityMask":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttBrushConverter.Convert(attrib.Value), AttBrushConverter.Usings));
                        break;
                    
                    //Point
                    case "RenderTransformOrigin":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttPointConverter.Convert(attrib.Value), AttPointConverter.Usings));
                        break;

                    //Size
                    case "RenderSize":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttSizeConverter.Convert(attrib.Value), AttSizeConverter.Usings));
                        break;

                    //Visibility
                    case "Visibility":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttVisibilityConverter.Convert(attrib.Value), AttVisibilityConverter.Usings));
                        break;
                    
                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // Meldungen ins Log schreiben, wenn Attribute nicht behandelt werden konnten
            foreach(ElemAttribute attrib in unhandled)
                Logging.Instance.AddMessage("Noch keine Attributbehandlung für: " + attrib.Name);

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            return ("UIElement" == tag);
        }
    }
}
