﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.NodeGenerators.BaseGenerators;
using Konverter.Statements;
using Konverter.CodeTree;
using Konverter.ConverterBase;
using Konverter.AttribValueConverters;

namespace Konverter.NodeGenerators.Generators
{
    /// <summary>
    /// Konkreter NodeGenerator zum Behandeln von Baumknoten des CSTree,
    /// die einen LinearGradientBrush repräsentieren.
    /// </summary>
    class GenLinearGradientBrush : BaseGenGradientBrush, ChildElementsAddHandler
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public GenLinearGradientBrush() : base()
        {
            tagName = "LinearGradientBrush";     // Name des Tags setzen, für das der Generator verantwortlich sein soll
            varBaseName = "linearGradientBrush"; // Basisname für Variablen des Typs "LienarGradientBrush"

            // benötigte Usings setzen
            usings = new List<UsingStmt>() { new UsingStmt("System.Windows.Media") };

            // Informationen über Collection-Attribute hinzufügen
            collectionAttribs.Add("GradientStops");
        }

        /// <summary>
        /// Generiert aus den übergebenen Attributen eine Liste von Baumknoten des CSTree,
        /// welche die Attributwerte beinhalten.
        /// </summary>
        /// <param name="attribs">Zu verarbeitende Attribute</param>
        /// <returns>Generierte Baumknoten-Liste</returns>
        public override List<PropertyValueNode> GenerateAttribNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();

            // Behandlung aller Attribute, welche direkt in LinearGradientBrush definiert sind
            foreach (ElemAttribute attrib in attribs)
            {
                switch (attrib.Name)
                {
                    //Point
                    case "StartPoint":
                    case "EndPoint":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttPointConverter.Convert(attrib.Value), AttPointConverter.Usings));
                        break;
                    
                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // Übergabe der noch unbehandelten Attribute an die Basisklasse und Verarbeitung von dieser
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Gibt eine Liste von Anweisungen zurück, welche in einem LinearGradientBrush
        /// definierte Kindknoten zu einem Elternknoten hinzufügen.
        /// </summary>
        /// <param name="parent">Elternknoten</param>
        /// <param name="children">Kind-Elementknoten des Elternknotens</param>
        /// <returns>Anweisungsliste zum Hinzufügen der Kinder zum Elter</returns>
        public List<Statement> GenerateAddCode(ChildElementNode parent, List<ChildElementNode> children)
        {
            List<Statement> addCode = new List<Statement>();

            // gehe Kindliste durch
            foreach (ChildElementNode child in children)
            {
                // behandle Kinder des Typs "GradientStop"
                if (child.NodeGenerator is GenGradientStop)
                    addCode.Add(new MethodCall(parent.VarName + ".GradientStops.Add", child.VarName));
                // für den Rest: Meldung "Unbehandelt" in das Log schreiben
                else
                    Logging.Instance.AddMessage("Keine Behandlung von " + parent.NodeGenerator.TagName + " für Kinder des Typs " + child.NodeGenerator.TagName);
            }

            return addCode;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if (this.tagName == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }
    }
}
