﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.ConverterBase;
using Konverter.CodeTree;
using Konverter.Statements;
using Konverter.NodeGenerators.BaseGenerators;
using Konverter.AttribValueConverters;

namespace Konverter.NodeGenerators.Generators
{
    /// <summary>
    /// Konkreter NodeGenerator zum Behandeln von Baumknoten des CSTree,
    /// die einen SolidColorBrush repräsentieren.
    /// Dabei wird die Schnittstelle "TextAsAttribHandler" implementiert,
    /// um XmlText über das Attribut "Color" behandeln zu lassen.
    /// </summary>
    class GenSolidColorBrush : BaseGenBrush, TextAsAttribHandler
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public GenSolidColorBrush() : base()
        {
            tagName = "SolidColorBrush";         // Name des Tags setzen, für das der Generator verantwortlich sein soll
            varBaseName = "solidColorBrush";     // Basisname für Variablen des Typs "SolidColorBrush"
            
            // benötigte Usings setzen
            usings = new List<UsingStmt>() { new UsingStmt("System.Windows.Media") };
        }

        /// <summary>
        /// Generiert aus den übergebenen Attributen eine Liste von Baumknoten des CSTree,
        /// welche die Attributwerte beinhalten.
        /// </summary>
        /// <param name="attribs">Zu verarbeitende Attribute</param>
        /// <returns>Generierte Baumknoten-Liste</returns>
        public override List<PropertyValueNode> GenerateAttribNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();
            
            // Behandlung aller Attribute, welche direkt in SolidColorBrush definiert sind
            foreach (ElemAttribute attrib in attribs)
            {
                switch (attrib.Name)
                {
                    //Color
                    case "Color":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttColorConverter.Convert(attrib.Value), AttColorConverter.Usings));
                        break;
                    
                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // Übergabe der noch unbehandelten Attribute an die Basisklasse und Verarbeitung von dieser
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Gibt den Namen des Attributs zurück, über welches XmlText zu behandeln ist (hier: "Color").
        /// </summary>
        /// <returns>Attributname</returns>
        public string GetTextAttribName()
        {
            return "Color";
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if (this.tagName == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }
    }
}
