﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.ConverterBase;
using Konverter.CodeTree;
using Konverter.NodeGenerators.BaseGenerators;
using Konverter.Statements;
using Konverter.AttribValueConverters;

namespace Konverter.NodeGenerators.Generators
{
    /// <summary>
    /// Konkreter NodeGenerator zum Behandeln von Baumknoten des CSTree,
    /// die eine TextBox repräsentieren.
    /// Dabei wird die Schnittstelle "TextAsAttribHandler" implementiert,
    /// um XmlText über das Attribut "Text" behandeln zu lassen.
    /// </summary>
    class GenTextBox : BaseGenTextBoxBase
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public GenTextBox() : base()
        {
            tagName = "TextBox";                 // Name des Tags setzen, für das der Generator verantwortlich sein soll
            varBaseName = "textBox";             // Basisname für Variablen des Typs "TextBox"
            
            // benötigte Usings setzen
            usings = new List<UsingStmt>() { new UsingStmt("System.Windows.Controls") };
        }

        /// <summary>
        /// Generiert aus den übergebenen Attributen eine Liste von Baumknoten des CSTree,
        /// welche die Attributwerte beinhalten.
        /// </summary>
        /// <param name="attribs">Zu verarbeitende Attribute</param>
        /// <returns>Generierte Baumknoten-Liste</returns>
        public override List<PropertyValueNode> GenerateAttribNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();
            
            // hier werden alle Attribute behandelt, die direkt in TextBox definiert sind
            foreach (ElemAttribute attrib in attribs)
            {
                switch (attrib.Name)
                {
                    //int
                    case "CaretIndex":
                    case "MaxLength":
                    case "MaxLines":
                    case "MinLines":
                    case "SelectionLength":
                    case "SelectionStart":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttIntConverter.Convert(attrib.Value)));
                        break;
                    
                    //string
                    case "SelectedText":
                    case "Text":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttStringConverter.Convert(attrib.Value)));
                        break;

                    //CharacterCasing
                    case "CharacterCasing":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttCharacterCasingConverter.Convert(attrib.Value), AttCharacterCasingConverter.Usings));
                        break;

                    //TextAlignment
                    case "TextAlignment":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttTextAlignmentConverter.Convert(attrib.Value), AttTextAlignmentConverter.Usings));
                        break;

                    //TextWrapping
                    case "TextWrapping":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttTextWrappingConverter.Convert(attrib.Value), AttTextWrappingConverter.Usings));
                        break;

                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // Übergabe der noch unbehandelten Attribute an die Basisklasse und Verarbeitung von dieser
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if (this.tagName == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }
    }
}
