﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Konverter.Statements;
using Konverter.ConverterBase;
using Konverter.CodeTree;

namespace Konverter.NodeGenerators
{
    /// <summary>
    /// Abstrakte Basisklasse für konkrete Node-Generatoren.
    /// Ein NodeGenerator ist für die Verwaltung von Baumknoten des CSTree für konkrete
    /// Typen wie Button, TextBox etc. zuständig.
    /// Ein NodeGenerator erlaubt einem Baumknoten eines konkreten Typs:
    /// 1.) Fortlaufend die Variablennamen des Typs zu generieren und zu speichern,
    /// 2.) Für den Typ aus einer Liste übergebener Attribute eine Liste von Attributknoten
    ///     zu erzeugen, die unter den jeweiligen Baumknoten gehangen werden.
    /// Um Letzteres zu erreichen, müsste theoretisch jeder NodeGenerator alle Properties behandeln,
    /// die in dem konkreten C#-Typ definiert sind. Die meisten Properties stammen aber nicht direkt 
    /// von diesem Typ, sondern von Basisklassen. Damit wäre eine direkte Implementierung durch die
    /// verschiedensten Node-Generatoren stark redundant und wartungs-/änderungsunfreundlich.
    /// Aufgrund dessen soll bei der Erstellung eines NodGenerator's für einen konkreten C#-Typ
    /// die C#-Hierarchie abgebildet werden, die dem Typ zugrunde liegt. D.h. es werden abstrakte
    /// Generatoren für Basisklassen erstellt, welche die Methode GenerateBaseNodes() des
    /// NodeGenerator's implementieren und welche für genau die Attribute zuständig sind, die
    /// auch im C# an dieser Stelle der Hierarchie definiert sind. Der konkrete NodeGenerator
    /// für den zu behandelnden Typ soll dann die Methode GenerateAttribNodes() implementieren und
    /// für nicht behandelte Attribute die Basisklasse aufrufen.
    /// Durch diese Hierarchie-Bildung ist es möglich, dass konkrete Node-Generatoren auf dieselbe
    /// Basisklasse zugreifen und ihre Attribute von dieser behandeln lassen. Durch diese Abbildung
    /// der C#-Struktur wird größtmöglich Redundanz vermieden.
    /// </summary>
    abstract class NodeGenerator
    {
        /// <summary>
        /// Name des Xml-Tags, das von diesem Generator behandelt werden soll.
        /// </summary>
        protected string tagName = default(string);
        /// <summary>
        /// Basisteil des Namens für von diesem Generator erzeugte Variablen.
        /// </summary>
        protected string varBaseName = default(string);
        /// <summary>
        /// Name der zuletzt erzeugten Variablen dieses Generators.
        /// </summary>
        protected string currentVarName = default(string);
        /// <summary>
        /// Zähler für Indizes der erzeugten Variablen.
        /// </summary>
        protected int varIndex = 0;
        /// <summary>
        /// Liste bereits verbrauchter Variablennamen.
        /// </summary>
        private List<string> usedNames = new List<string>();
        /// <summary>
        /// Liste der von einem konkreten NodeGenerator benötigten Usings.
        /// </summary>
        protected List<UsingStmt> usings;
        /// <summary>
        /// Liste die angibt, welche Attribute als Collection aufzufassen sind.
        /// </summary>
        protected List<String> collectionAttribs = new List<String>();

        /// <summary>
        /// Gibt den Namen der von diesem NodeGenerator verarbeiteten Tags zurück.
        /// </summary>
        public string TagName
        {
            get { return this.tagName; }
        }

        /// <summary>
        /// Gibt den nächsten Variablenindex zurück.
        /// </summary>
        private int NextIndex
        {
            get
            {
                varIndex++;
                return varIndex;
            }
        }

        /// <summary>
        /// Gibt den aktuell gespeicherten Variablennamen für diesen Generator zurück.
        /// </summary>
        public string CurrentVarName
        {
            get { return currentVarName; }
        }

        /// <summary>
        /// Erzeugt den nächsten Variablennamen (verbraucht dabei mind. den nächsthöheren Index).
        /// </summary>
        private string NextVarName
        {
            get {
                string varName = default(string);
                bool isUnique = false;
                // erzeuge den nächsten Variablennamen und prüfe ihn auf Eindeutigkeit
                // mit den bereits erstellten Variablen
                while (!isUnique)
                {
                    isUnique = true;
                    varName = varBaseName + NextIndex;
                    if(usedNames.Contains(varName))
                        isUnique = false;
                }
                return varName;
            }
        }

        /// <summary>
        /// Gibt die Liste der von diesem NodeGenerator benötigten Usings zurück.
        /// </summary>
        public List<UsingStmt> ElementUsings
        {
            get { return usings; }
        }

        /// <summary>
        /// Gibt zurück, welche Attribute/Properties als Collection aufgefasst werden können.
        /// </summary>
        public List<String> CollectionAttribs
        {
            get { return collectionAttribs; }
        }


        /// <summary>
        /// Konstruktor.
        /// </summary>
        public NodeGenerator() { }

        /// <summary>
        /// Setzt den aktuell verwendeten Variablennamen dieses Generators anhand der übergebenen Attributliste.
        /// Enthält diese das Attribut "Name" und wurde noch keine Variable mit diesem Namen erzeugt, so wird 
        /// dieses verwendet, ansonsten der nächste automatisch erzeugte Variablenname
        /// (Basisname + nächster Variablenindex).
        /// </summary>
        /// <param name="attribs">Zu durchsuchende Attribute</param>
        /// <returns>Generierter Variablenname</returns>
        public string GenCurrentVarName(List<ElemAttribute> attribs)
        {
            string name = default(string);

            if (attribs != null)
            {
                // suche nach dem Attribut "Name"
                foreach (ElemAttribute att in attribs)
                {
                    if (att.Name == "Name" && !usedNames.Contains(att.Value))
                    {
                        name = att.Value.Trim();
                        break;
                    }
                }
            }
            
            // setze den Namen auf das Name-Attribut oder auf den nächsten automatisch generierten Namen
            this.currentVarName = (name != default(string)) ? name : NextVarName;
            usedNames.Add(this.currentVarName);
            return this.currentVarName;
        }

        /// <summary>
        /// Setzt den aktuellen Variablennamen auf den Wert des übergebenen Attributs,
        /// welches (um Sinn zu machen) den Namen der Variablen angeben sollte.
        /// Wurde bereits eine Variable mit diesem Namen erzeugt, so wird der nächste
        /// Variablenname automatisch generiert.
        /// </summary>
        /// <param name="nameAttrib">Attribut, dessen Wert als Variablenname verwendet werden soll</param>
        /// <returns>Erzeugten Variablennamen</returns>
        public string GenCurrentVarName(ElemAttribute nameAttrib)
        {
            this.currentVarName = (nameAttrib != null && !usedNames.Contains(nameAttrib.Value)) ? nameAttrib.Value : NextVarName;
            usedNames.Add(this.currentVarName);
            return this.currentVarName;
        }

        /// <summary>
        /// Generiert den nächsten Variablennamen und gibt ihn zurück.
        /// </summary>
        /// <returns>Erzeugten Variablennamen</returns>
        public string GenCurrentVarName()
        {
            this.currentVarName = NextVarName;
            usedNames.Add(this.currentVarName);
            return this.currentVarName;
        }

        /// <summary>
        /// Setzt den Zähler für Variablenindizes dieses Generators zurück,
        /// leert die Liste verbrauchter Variablen und entfernt den zuletzt
        /// erzeugten Variablennamen.
        /// </summary>
        public void ResetIndexCounter()
        {
            this.varIndex = 0;
            this.usedNames.Clear();
            this.currentVarName = default(string);
        }

        /// <summary>
        /// Generiert die Anweisung zur Erzeugung eines neuen Objekts vom 
        /// in diesem (konkreten) NodeGenerator gespeicherten Typ in C#.
        /// </summary>
        /// <param name="varName">Variablenname für das neue Objekt</param>
        /// <returns>Code für die Erstellung eines neuen Objekts</returns>
        public virtual Assignment GenNewObjectStmt(string varName)
        {
            return new Assignment(tagName + " " + varName, "new " + tagName + "()");
        }

        /// <summary>
        /// Abstrakte Methode. Diese soll von konkreten Node-Generatoren implementiert werden.
        /// Aus den übergebenen Attributen soll dabei eine Liste von Wert-Attributknoten des
        /// CSTree erstellt werden, die dem aufrufenden Baumknoten hinzugefügt werden.
        /// </summary>
        /// <param name="attribs">Attributliste, für die Baumknoten zu erstellen sind</param>
        /// <returns>Aus den Attributen erzeugter Code</returns>
        public abstract List<PropertyValueNode> GenerateAttribNodes(List<ElemAttribute> attribs);

        /// <summary>
        /// Abstrakte Methode. Diese soll in ebenfalls abstrakten Vererbungen von NodeGenerator
        /// implementiert werden, bei denen es sich noch nicht um konkrete Node-Generatoren 
        /// handelt, sondern um Basisklassen von diesen.
        /// Dadurch soll es diesen abstrakten Oberklassen von konkreten Implementierungen
        /// eines Generators ermöglicht werden, gemeinsam genutzte Attribute in Baumknoten
        /// umzuwandeln und den vererbten Klassen über einen einfachen Methodenaufruf 
        /// zur Verfügung zu stellen.
        /// </summary>
        /// <returns>Erzeugte Baumknoten-Liste</returns>
        protected abstract List<PropertyValueNode> GenerateBaseNodes(List<ElemAttribute> attribs);

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public abstract bool HasTagInHierarchy(string tag);
    }
}
