﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Konverter.NodeGenerators.Generators;

namespace Konverter.NodeGenerators
{
    /// <summary>
    /// Managerklasse für konkrete Node-Generatoren. Vor allem soll damit erreicht werden, dass
    /// zu einem übergebenen Xml-Tag/Xml-Node der jeweils zuständige NodeGenerator ermittelt werden kann.
    /// Die Klasse ist nach dem Singleton-Entwurfsmuster implementiert, sodass nicht jede Klasse, die
    /// hierauf Zugriff haben will, eine eigene Instanz erzeugen muss. Ebenso lassen sich dadurch
    /// statische Attribute und Methoden vermeiden (bis auf das Singleton an sich).
    /// </summary>
    class NodeGeneratorManager
    {
        # region Statische Singleton Mitglieder
        /// <summary>
        /// Statische Singleton-Instanz der Klasse.
        /// </summary>
        static NodeGeneratorManager inst = null;

        /// <summary>
        /// Liefert die statische Singleton-Instanz der Klasse zurück.
        /// </summary>
        public static NodeGeneratorManager Instance
        {
            get
            {
                if (inst == null)
                    inst = new NodeGeneratorManager();
                return inst;
            }
        }
        #endregion

        /// <summary>
        /// Dictionary verfügbarer konkreter Node-Generatoren.
        /// Dabei gilt das verwaltete Tag als Schlüssel und der Generator selbst als Wert.
        /// </summary>
        Dictionary<String,NodeGenerator> nodeGenerators = null;

        /// <summary>
        /// Konstruktor. private aufgrund der Singleton-Benutzung.
        /// Hier werden alle Node-Generatoren festgelegt, die von diesem Manager verwaltet werden sollen.
        /// </summary>
        private NodeGeneratorManager()
        {
            nodeGenerators = new Dictionary<String,NodeGenerator>();
            addGenerators(new List<NodeGenerator>(){
                new GenButton(),
                new GenCheckBox(),
                new GenComboBox(),
                new GenComboBoxItem(),
                new GenGradientStop(),
                new GenGrid(),
                new GenGroupBox(),
                new GenLabel(),
                new GenLinearGradientBrush(),
                new GenListBox(),
                new GenListBoxItem(),
                new GenMenu(),
                new GenMenuItem(),
                new GenRadioButton(),
                new GenSolidColorBrush(),
                new GenStackPanel(),
                new GenTabControl(),
                new GenTabItem(),
                new GenTextBox(),
                new GenToolBar()
            });
        }

        /// <summary>
        /// Füge die zu verwaltenden Node-Generatoren zum Dictionary hinzu.
        /// </summary>
        /// <param name="generators">Zu verwaltende Node-Generatoren</param>
        private void addGenerators(IEnumerable<NodeGenerator> generators){
            foreach (NodeGenerator gen in generators)
                nodeGenerators.Add(gen.TagName, gen);
        }

        /// <summary>
        /// Liefert den zum übergebenen Tag zuständigen NodeGenerator zurück.
        /// </summary>
        /// <param name="tag">Xml-Tag</param>
        /// <returns>NodeGenerator, der für das Xml-Tag zuständig ist oder null, wenn sich keiner findet</returns>
        public NodeGenerator GetGeneratorForTag(string tag)
        {
            NodeGenerator gen = null;
            nodeGenerators.TryGetValue(tag, out gen);
            return gen;
        }

        /// <summary>
        /// Liefert den zum übergebenen XmlNode zuständigen Node-Generator zurück.
        /// </summary>
        /// <param name="node">Xml-Knoten</param>
        /// <returns>NodeGenerator, der für den XmlNode zuständig ist oder null, wenn sich keiner findet.</returns>
        public NodeGenerator GetGeneratorForXmlNode(XmlNode node)
        {
            return GetGeneratorForTag(node.Name);
        }

        /// <summary>
        /// Setzt die Zähler für Variablennamen-Indizes aller Node-Generatoren zurück.
        /// </summary>
        public void ResetIndexCounters()
        {
            foreach (NodeGenerator gen in this.nodeGenerators.Values)
                gen.ResetIndexCounter();
        }
    }
}
